// Robot Vision勉強会 sample6.cpp
// 2017-11-30
// フィルタ処理のサンプルプログラム 

#include <opencv2/imgproc.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/core/core.hpp>
#include <iostream>

// 膨張（４近傍)
void myDilate(cv::Mat img_src, cv::Mat& img_dst)
{
  int height = img_src.rows;       // 画像の高さ[pixel]
  int width  = img_src.cols;       // 画像の幅[pixel] 
  
  for (int y =1; y < height-1; y++) {
    for (int x = 1; x < width-1; x++) {
      if ((img_src.data[y * width + x] == 255)
	  || (img_src.data[y * width + (x-1)] == 255)
	  || (img_src.data[y * width + (x+1)] == 255)
	  || (img_src.data[(y-1) * width + x] == 255) 
	  || (img_src.data[(y+1) * width + x] == 255)) {
	img_dst.data[y * width + x] = 255;
      }
      else {
	img_dst.data[y * width + x] = 0;
      }
    }
  }
}


// 収縮（４近傍)
void myErode(cv::Mat img_src, cv::Mat& img_dst)
{
  int height = img_src.rows;       // 画像の高さ[pixel]
  int width  = img_src.cols;       // 画像の幅[pixel]

  for (int y =1; y < height-1; y++) {
    for (int x = 1; x < width-1; x++) {
      if ((img_src.data[y * width + x] == 0)
	  || (img_src.data[y * width + (x-1)] == 0)
	  || (img_src.data[y * width + (x+1)] == 0)
	  || (img_src.data[(y-1) * width + x] == 0)
	  || (img_src.data[(y+1) * width + x] == 0)) {
	img_dst.data[y * width + x] = 0;
      }
      else {
	img_dst.data[y * width + x] = 255;
      }
    }
  }
}

int main()
{
  cv::Mat src;

  std::string WINDOW_NAME_ORG        = "Original image";
  std::string WINDOW_NAME_BINARY     = "Binary image";
  std::string WINDOW_NAME_DILATE     = "Dilate";
  std::string WINDOW_NAME_ERODE      = "Erode";
  std::string WINDOW_NAME_MY_DILATE  = "My Dilate";
  std::string WINDOW_NAME_MY_ERODE   = "My Erode";
  
  
  // 画像のロード
  src = cv::imread("winkit2.jpg", cv::IMREAD_COLOR);
  if(src.empty()) {
    std::cerr << "Failed to open image file." << std::endl;
    return -1;
  }

  int height = src.rows;       // 画像の高さ[pixel]
  int width  = src.cols;       // 画像の幅[pixel]
  int step   = src.step;       // 1行のチャンネル総数
  int c      = src.channels(); // チャンネル数   

  cv::Mat tmp = src.clone();
  cv::Mat gray_img, binary_img;
  
  // ウィンドウの生成
  cv::namedWindow(WINDOW_NAME_ORG,       CV_WINDOW_AUTOSIZE);
  cv::namedWindow(WINDOW_NAME_BINARY,    CV_WINDOW_AUTOSIZE);
  cv::namedWindow(WINDOW_NAME_DILATE,    CV_WINDOW_AUTOSIZE);
  cv::namedWindow(WINDOW_NAME_MY_DILATE, CV_WINDOW_AUTOSIZE);
  cv::namedWindow(WINDOW_NAME_ERODE,     CV_WINDOW_AUTOSIZE);
  cv::namedWindow(WINDOW_NAME_MY_ERODE,  CV_WINDOW_AUTOSIZE);

  
  while (true){
    src.copyTo(tmp);
    cvtColor(tmp, gray_img, CV_BGR2GRAY);

    // 2値化
    int thresh = 200;
    //cv::threshold(gray_img, binary_img, thresh, 255, cv::THRESH_BINARY);
    cv::threshold(gray_img, binary_img, 0, 255, cv::THRESH_BINARY| cv::THRESH_OTSU);
    cv::imshow(WINDOW_NAME_BINARY, binary_img);

    cv::Mat dilate_img;
    cv::Mat erode_img;  
    cv::Mat my_dilate_img  = cv::Mat::zeros(height, width, CV_8U);
    cv::Mat my_erode_img   = cv::Mat::zeros(height, width, CV_8U);
    cv::Mat element4 = (cv::Mat_<uchar>(3,3) << 0,1,0,1,1,1,0,1,0);
    cv::dilate(binary_img, dilate_img, element4, cv::Point(-1,-1), 1);
    cv::erode(binary_img,  erode_img,  element4, cv::Point(-1,-1), 1);
    
    myDilate(binary_img, my_dilate_img);
    myErode(binary_img, my_erode_img);

    cv::imshow(WINDOW_NAME_ORG, tmp);
    cv::imshow(WINDOW_NAME_DILATE,    dilate_img);
    cv::imshow(WINDOW_NAME_MY_DILATE, my_dilate_img);
    cv::imshow(WINDOW_NAME_ERODE,     erode_img);
    cv::imshow(WINDOW_NAME_MY_ERODE,  my_erode_img);
    
    cv::waitKey(1);
  }

  return 0;
}
